package io.openems.edge.controller.ess.timeofusetariff.discharge;

import org.osgi.service.metatype.annotations.AttributeDefinition;
import org.osgi.service.metatype.annotations.ObjectClassDefinition;

@ObjectClassDefinition(//
		name = "Controller Ess Time-Of-Use Tariff Discharge", //
		description = "Optimize behaviour of an ESS in combination with a Time-Of-Use (ToU) Tariff. Applies to German legislation.")
@interface Config {

	@AttributeDefinition(name = "Component-ID", description = "Unique ID of this Component")
	String id() default "ctrlEssTimeOfUseTariffDischarge0";

	@AttributeDefinition(name = "Alias", description = "Human-readable name of this Component; defaults to Component-ID")
	String alias() default "";

	@AttributeDefinition(name = "Is enabled?", description = "Is this Component enabled?")
	boolean enabled() default true;

	@AttributeDefinition(name = "Ess-ID", description = "ID of Ess device.")
	String ess_id();

	@AttributeDefinition(name = "Mode", description = "Set the type of mode.")
	Mode mode() default Mode.AUTOMATIC;

	@AttributeDefinition(name = "Risk level of the customer", description = "" //
			+ "Low Risk: Less dependence on predictions; Energy in battery should always be used during the night. " //
			+ "High Risk: High dependence on predictions; Battery is scheduled to discharge completely based on predictions.")
	DelayDischargeRiskLevel delayDischargeRiskLevel() default DelayDischargeRiskLevel.MEDIUM;

	@AttributeDefinition(name = "Fallback Morning-Hour", description = "Fallback for calculation to stop at this hour")
	int maxStartHour() default 8;

	@AttributeDefinition(name = "Fallback Evening-Hour", description = "Fallback for calculation to start at this hour")
	int maxEndHour() default 17;

	@AttributeDefinition(name = "Ess target filter", description = "This is auto-generated by 'Ess-ID'.")
	String ess_target() default "(enabled=true)";

	String webconsole_configurationFactory_nameHint() default "Controller Ess Time-Of-Use Tariff Discharge [{id}]";

}
