package io.openems.edge.goodwe.batteryinverter;

import org.osgi.service.metatype.annotations.AttributeDefinition;
import org.osgi.service.metatype.annotations.ObjectClassDefinition;

import io.openems.edge.goodwe.GoodWeConstants;
import io.openems.edge.goodwe.common.enums.ControlMode;
import io.openems.edge.goodwe.common.enums.EnableDisable;
import io.openems.edge.goodwe.common.enums.FeedInPowerSettings;
import io.openems.edge.goodwe.common.enums.SafetyCountry;

@ObjectClassDefinition(//
		name = "GoodWe Battery-Inverter", //
		description = "GoodWe Battery Inverter.")

@interface Config {
	@AttributeDefinition(name = "Component-ID", description = "Unique ID of this Component")
	String id() default "batteryInverter0";

	@AttributeDefinition(name = "Alias", description = "Human-readable name of this Component; defaults to Component-ID")
	String alias() default "";

	@AttributeDefinition(name = "Is enabled?", description = "Is this Component enabled?")
	boolean enabled() default true;

	@AttributeDefinition(name = "Control mode", description = "Sets the Control mode")
	ControlMode controlMode() default ControlMode.SMART;

	@AttributeDefinition(name = "Modbus-ID", description = "ID of Modbus bridge.")
	String modbus_id() default "modbus0";

	@AttributeDefinition(name = "Modbus Unit-ID", description = "The Unit-ID of the Modbus device.")
	int modbusUnitId() default GoodWeConstants.DEFAULT_UNIT_ID;

	@AttributeDefinition(name = "Modbus target filter", description = "This is auto-generated by 'Modbus-ID'.")
	String Modbus_target() default "(enabled=true)";

	@AttributeDefinition(name = "Country Setting", description = "Selection of feed in country")
	SafetyCountry safetyCountry() default SafetyCountry.GERMANY;

	@AttributeDefinition(name = "Mppt For Shadow Enable or Disable", description = "Selection the activation of Mppt tracker")
	EnableDisable mpptForShadowEnable() default EnableDisable.ENABLE;

	@AttributeDefinition(name = "Enable/Disable Backup Power", description = "In case of Off-Grid case backup power will be activated")
	EnableDisable backupEnable() default EnableDisable.ENABLE;

	@AttributeDefinition(name = "Enable Feed In To Grid", description = "Enable-disable feed in to grid")
	EnableDisable feedPowerEnable() default EnableDisable.DISABLE;

	@AttributeDefinition(name = "Feed In To Grid Power", description = "Fixed feed in to grid power (Only positive [0 - 10_000])")
	int feedPowerPara() default 0;

	@AttributeDefinition(name = "Feed In Power Selection", description = "This is the selection of inverter power settings")
	FeedInPowerSettings setfeedInPowerSettings() default FeedInPowerSettings.UNDEFINED;

	String webconsole_configurationFactory_nameHint() default "GoodWe Battery Inverter [{id}]";
}
